# -*- coding: utf-8 -*-

"""
 (c) 2019 - Copyright Red Hat Inc

 Authors:
   Pierre-Yves Chibon <pingou@pingoured.fr>

"""

from __future__ import print_function, unicode_literals

import logging

from pagure.lib.model import Project
from pagure.lib.model_base import BASE

import sqlalchemy as sa
from sqlalchemy.orm import backref, relation

_log = logging.getLogger(__name__)


# DB Model


class PagureAnitya(BASE):
    """Stores information about the monitoring of a project in anitya.

    Table -- pagure_anitya
    """

    __tablename__ = "pagure_anitya"

    id = sa.Column(sa.Integer, primary_key=True)
    project_id = sa.Column(
        sa.Integer,
        sa.ForeignKey("projects.id", onupdate="CASCADE", ondelete="CASCADE"),
        nullable=False,
        unique=True,
        index=True,
    )

    anitya_status = sa.Column(
        sa.String(255), nullable=False, unique=False, default="no-monitoring"
    )

    project = relation(
        "Project",
        remote_side=[Project.id],
        backref=backref(
            "anitya",
            cascade="delete, delete-orphan",
            single_parent=True,
            uselist=False,
        ),
    )


class PagureOrphanReason(BASE):
    """Stores information about the orphaning reason.

    Table -- pagure_orphan_reason

    Attributes:
        project_id: Id of orphaned project
        reason: Reason for orphaning
        reason: Any additional info for orphaning
    """

    __tablename__ = "pagure_orphan_reason"

    id = sa.Column(sa.Integer, primary_key=True)
    project_id = sa.Column(
        sa.Integer,
        sa.ForeignKey("projects.id", onupdate="CASCADE", ondelete="CASCADE"),
        nullable=False,
        unique=True,
        index=True,
    )

    reason = sa.Column(sa.String(255), nullable=True, unique=False)

    reason_info = sa.Column(sa.String(255), nullable=True, unique=False)

    project = relation(
        "Project",
        remote_side=[Project.id],
        backref=backref(
            "orphan_reason",
            cascade="delete, delete-orphan",
            single_parent=True,
            uselist=False,
        ),
    )


class PagureBZOverride(BASE):
    """Stores information about default assignees (Fedora vs. EPEL)

    Table -- pagure_bzoverride
    """

    __tablename__ = "pagure_bzoverride"

    id = sa.Column(sa.Integer, primary_key=True)
    project_id = sa.Column(
        sa.Integer,
        sa.ForeignKey("projects.id", onupdate="CASCADE", ondelete="CASCADE"),
        nullable=False,
        unique=True,
        index=True,
    )

    epel_assignee = sa.Column(sa.String(255), unique=False, nullable=True)
    fedora_assignee = sa.Column(sa.String(255), unique=False, nullable=True)

    project = relation(
        "Project",
        remote_side=[Project.id],
        backref=backref(
            "bzoverride",
            cascade="delete, delete-orphan",
            single_parent=True,
            uselist=False,
        ),
    )
